#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar  6 10:11:09 2021

@author: marcel
"""

from astropy.io import fits
import glob
import os

from astropy.time import Time
from astroquery.jplhorizons import Horizons
from astropy import units as u
from astropy.coordinates import SkyCoord 


def updateobjdes(foldername, olddesig, newdesig):
    """
    This function is used to update the name of the object for all files
    obtaiend with MUSCAT and pre-processed initially
    Parameters
    ----------
    foldername : STRING
        the main folder containing the folder with object data.
    olddesig : STRING
        the initial designation for the object.
    newdesig : STRING
        The new designation.For temporary designations use only this format
        'YYYY XXNNN' ex: '2003 AV34'

    Returns
    -------
    The object folder wil be renamed and the header keyword 'OBJECT' will be 
    updated
    Ussage:updateobjdes('/home/marcel/990105/', 2003AV34B, '2003 AV34') 

    """    
    os.system('mv ' + foldername + olddesig + ' '  + foldername + newdesig)
    
    for filt in ['g', 'r', 'i', 'z_s']:
        if (os.path.isdir(foldername + newdesig + '/' + filt + '/')):
            filenames = glob.glob(foldername + newdesig + '/' 
                                                + filt + '/*.fits')
            for fname in filenames:
                with fits.open(fname, mode='update') as hdul:
                    hdul[0].header['OBJECT'] = newdesig
                    hdul.flush()  # changes are written back to original.fits
                    hdul.close()
    return


def updatecoordinates(foldername, objdesig):
    """
    This function updates the RA and DEC positions by performing 
    a JPL Horizon query for the object designation found in the header

    Parameters
    ----------
    foldername : STRING
        Main folder - containing all observations of that night
    objdesig : STRING
        object designation (name of the subfolder in the main folder)

    Returns
    -------
    Updates the RA and DEC keywords in the header

    """
    for filt in ['g', 'r', 'i', 'z_s']:
        if (os.path.isdir(foldername + objdesig + '/' + filt + '/')):
            filenames = glob.glob(foldername + objdesig + '/' 
                                                + filt + '/*.fits')
            if not('ra' in vars()):
                filenames.sort()
                data, hd = fits.getdata(filenames[int(len(filenames)/2)], header=True)
                t = Time(hd['DATE-OBS'] + 'T' + hd['EXP-STRT'])
                obj = Horizons(hd['OBJECT'], epochs=t.jd, location='954')
                eph = obj.ephemerides()
                c = SkyCoord(ra=float(eph['RA'])*u.degree,
                                     dec=float(eph['DEC'])*u.degree, frame='icrs')
                ra = c.to_string('hmsdms').split(' ')[0].replace('h',':').replace('m',':').replace('s','')
                dec = c.to_string('hmsdms').split(' ')[1].replace('d',':').replace('m',':').replace('s','')
        
            for fname in filenames:
                with fits.open(fname, mode='update') as hdul:
                    hdul[0].header['RA'] = ra
                    hdul[0].header['DEC'] = dec
                    hdul.flush()  # changes are written back to original.fits
                    hdul.close()
    return  
    

def applyskysub(foldername, objdesig):

    from pyraf import iraf
    iraf.unlearn('imarith')

    for filt in ['g', 'r', 'i', 'z_s']:
        currfolder = foldername + objdesig + '/' + filt + '/'
        if (os.path.isdir(currfolder)):
            filenames = glob.glob(currfolder + 'proc_*.fits')
            os.chdir(currfolder)
            from pyraf import iraf
            iraf.unlearn('imarith')
            for fname in filenames:
                fnamenew = 'p' + fname.split('/')[-1][13:]
                fnameini = fname.split('/')[-1]
                os.system('astnoisechisel ' +  fnameini + ' -h0 -q -otest.fits')
                iraf.imarith(currfolder+fnameini, '-', currfolder + 
                             'test.fits[SKY]', currfolder + fnamenew)
                os.system('rm ' + fnameini)
                os.system('rm test.fits')
                    
    

# updateobjdes('/home/marcel/990105/', '35107b', '35107')
# updatecoordinates('/home/marcel/990105/', '35107')
applyskysub('/home/marcel/990105/', '35107')    
          
    
