#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Mar 25 18:47:43 2021

@author: marcel
"""




import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import scipy as sp
from scipy import signal
from astropy.time import Time
from glob import glob
import os.path
import math



def ppdat2csv(fname):
    """
    This function converts the ".dat" files generated by the photometry 
    pipeline to ".csv" files, which can be loaded with panda data frame 

    Parameters
    ----------
    fname : file path
        Path to the ".dat" file generated by the photometry pipeline file
        In order to have the corect header, please review the format which
        which must contain the following fields
        "filename,JD,mag,sig,ra,dec,OCRA,OCDEC,MRAOFF,MDECOFF,EXPTime,ZP,ZPsig,instmag,instsig,Catalog,Filter,SextFlag,Instrum,Method,FWHM"
        Example:
            '/home/marcel/200316/137170/g/photometry_137170__1999_HF1_.dat'

    Returns
    -------
    ".csv" file located in the same path as ".dat" file

    """
    if not(os.path.isfile(fname)):  # raise error if file doesn't exists
        raise NameError("photom...dat file doesn't exists")
        
    fid = open(fname[:-4] + '.csv','w')  # open the new csv file
    # Write the header of the csv file
    fid.write("filename,JD,mag,sig,ra,dec,OCRA,OCDEC,MRAOFF,MDECOFF,EXPTime,ZP,ZPsig,instmag,instsig,Catalog,Filter,SextFlag,Instrum,Method,FWHM")
    # For each line in the photometric file
    for obs in open(fname, 'r').readlines():
        if '#' in obs:  # ignore the lines commented
            continue
        # convert to csv format and append to the new file
        fid.write("\n" + ",".join(obs.split()))
    fid.close() # close the csv file
    return  # END ppdat2csv function



def getphotfilename(foldername):
    """
    This function finds the name of the file containing the photometric data
    for the asteroids, which was reported by photometry pipeline
    It also transforms the photometric files ".dat" in the ".csv" files

    Parameters
    ----------
    foldername : folder path
        Path to the folder containing the ".dat" files
        Example:
            '/home/marcel/200316/137170/g/'
    Returns
    -------
    photfile : string
        Name of the file containg the asteroid photometric data
        without any extension
        Example:
            'photometry_137170__1999_HF1_'
    """
    photfile = ''  # output value if not file is found
    # get the name of the photometric files (should be the asteroid and 
    # the control star)
    photfiles = glob(foldername + 'phot*.dat')
    for fname in photfiles:
        # run the ".dat" to ".csv" transformation
        if not(os.path.isfile(fname[:-4] + '.csv')): 
            ppdat2csv(fname)
        fname = fname.split('/')[-1] # retreive just the name of the file
        # keep the asteroid file
        if not(fname == 'photometry_Control_Star.dat'): 
            photfile = fname[:-4]  # remove the extension
    return photfile  # END getphotfilename
 

########################################################

def computecolor(data1, data2):
        
    mytolerance = 5/3600/24
    for JD1, mag1, emag1 in data1[:,0:3]:
        for JD2, mag2, emag2 in data2[:,0:3]:
            if np.abs(JD1 - JD2) < mytolerance :
                col = np.round(mag1 - mag2, 4)
                ecol = np.round(np.sqrt(emag1**2 + emag2**2)/2, 4)
                if not('out' in locals()):
                    out = [[JD1, col, ecol]]
                else:
                    out = np.append(out,[[JD1, col, ecol]], axis=0)
                            
    if not('out' in locals()):
        raise NameError("Unable to compute the color, no simultaneous observations")
            
    return out


def reject_outliers(data, m=2):
    madall = np.median(np.abs(data - np.median(data)))
    aux = sp.signal.medfilt(data, 9)
    mad = np.median(np.abs(data - aux))
    return (abs(data - aux) < m * mad) & (abs(data - aux) < m * madall) 


def from_jd_convert(jd):
    t = Time(jd, format='jd', scale='utc')
    t.format = 'iso'
    return t.value


class datafilter:
    # Load the data for the filter and organize it
    def __init__(self, fname):
        if(os.path.exists(fname)):
            df = pd.read_csv(fname)
            df = df.sort_values(by=['JD'])
            
            data = df[['JD','mag','sig','ra','dec']].to_numpy()
            data[:,0] = (data[:,0] - data[0,0]) * 24
            
            # Report data as a numpy array
            self.data = data
            self.fname = fname
            self.JDstart = df.JD[0]
            # Prepare the output
            magfiltered = sp.signal.medfilt(data[:, 1], 9)  # filtered signal
            # Remove the first and the last points on filtered signal
            magfiltered = magfiltered[4:-4]
            # Save the variables
            self.data = data  # numpy array data
            self.filtername = fname.split('/')[-2]  # filtername
            self.ObjDesig = fname.split('/')[-3]  # objname
            # Observing time
            self.TotalTimeObs = np.round(data[-1, 0] - data[0, 0], 4)
            self.StartTimeObs = from_jd_convert(df.JD[0]).split(' ')[-1]
            # Mean value, Amplitude, and aproximative error
            self.MeanVal = np.round(data[:,1].mean(), 4)
            self.AmplMax = np.round(magfiltered.max() - magfiltered.min(), 4)
            self.MedErrPerObs = np.round(np.median(data[:,2]), 4)
            self.NImags = len(data[:,1]);
        else:            
            self.ObjDesig = fname.split('/')[-3]  # filtername        
            self.dict = np.nan
           
            self.data = np.nan
            self.fname = np.nan
            # Observing time
            self.TotalTimeObs = np.nan
            self.StartTimeObs = np.nan
            # Mean value, Amplitude, and aproximative error
            self.MeanVal = np.nan
            self.AmplMax = np.nan
            self.MedErrPerObs = np.nan
            self.NImags = np.nan

    def doplot(self, foldername = None):
        if(~np.isnan(self.MeanVal)):
            plt.figure()
            plt.plot(self.data[:, 0], self.data[:, 1], '*b')
            plt.plot(self.data[:, 0][4:-4], sp.signal.medfilt(self.data[:, 1], 9)[4:-4], '*-r')
            # plt.grid()        
            plt.ylim(self.MeanVal - self.AmplMax*0.75, self.MeanVal + self.AmplMax*0.75)
            plt.xlabel('time [h]', fontsize=14)
            plt.ylabel('Magnitude[mag]', fontsize=14)
            plt.title(self.ObjDesig + ' observed with ' + self.filtername + \
                      ' filter', fontsize=14)
            plt.gca().invert_yaxis()
            if(foldername):
                plt.savefig(foldername + self.ObjDesig + '_' +self.filtername \
                            + '.png', dpi=100)
                os.system('cp ' + self.fname + ' ' + foldername + 
                          self.ObjDesig + '_' +self.filtername + '.csv')
                plt.close()
        
class color:
    def __init__(self, f1, f2, ObjDesig='', ColName='f1mf2'):
        if((~np.isnan(f1.MeanVal)) & (~np.isnan(f2.MeanVal))):
            
            data1 = pd.read_csv(f1.fname).sort_values(by=['JD'])[['JD','mag','sig','ra','dec']].to_numpy()
            data2 = pd.read_csv(f2.fname).sort_values(by=['JD'])[['JD','mag','sig','ra','dec']].to_numpy()
            
            out = computecolor(data1,data2)
            
            
            self.ObjDesig = ObjDesig
            self.ColName  = ColName 
            self.rawcolor = out
            colorfiltered = sp.signal.medfilt(out[:, 1], 9)
            colorfiltered = colorfiltered[4:-4]
            self.MedColor = np.round(np.median(self.rawcolor[:,1]), 4)
            self.AvgColor = np.round(np.mean(self.rawcolor[:,1]), 4)
            self.ColorErr = np.round( np.sqrt(np.sum(np.power(self.rawcolor[:,2],2))) \
                            /len(self.rawcolor[:,2]),  4)
            self.ColorErr2 = np.std(self.rawcolor[:,1])/np.sqrt(len(self.rawcolor[:,1]))
            self.MedErrPerObs = np.round(np.median(self.rawcolor[:,2]), 4)
            self.AmplMax = np.round(colorfiltered.max() - colorfiltered.min(), 4)
        else:
            self.rawcolor = np.nan
            self.MedColor = np.nan
            self.AvgColor = np.nan
            self.ColorErr = np.nan
            self.ColorErr2 = np.nan
            self.MedErrPerObs = np.nan
            self.AmplMax = np.nan
            

    def doplot(self,foldername=None):
        if ~np.isnan(self.MedColor):
            plt.figure()
            t = (self.rawcolor[:,0] - self.rawcolor[0,0]) * 24
            colorfiltered = sp.signal.medfilt(self.rawcolor[:, 1], 9)
            plt.errorbar(t, self.rawcolor[:,1], yerr=self.rawcolor[:,2],
                         fmt ='*b', ecolor=(0.7,0.7,0.7))
            plt.plot(t[4:-4], colorfiltered[4:-4],'-r')
            #plt.grid()
            plt.xlabel('t[h]', fontsize=14)
            plt.ylabel('Color[mag]', fontsize=14)
            plt.title(self.ColName.replace('m','-') + ' of ' + self.ObjDesig, fontsize=14)
            if(foldername):
                plt.savefig(foldername + self.ObjDesig + '_' +self.ColName \
                            + '.png', dpi=100)
                plt.close()
            
def  prepareinforeport (foldername) :
    reportfolder = '/'.join(foldername.split('/')[:-2]) + '/Report_' + \
                   foldername.split('/')[-3] + '/'
    if not(os.path.isdir(reportfolder)):
        os.mkdir(reportfolder)
        
    os.system('cat ' + foldername + 'SelectData.py >> ' 
              + reportfolder + 'SelectData.py')
    
    photfname = getphotfilename(foldername + 'r/')
    g = datafilter(foldername + 'g/' + photfname + '_check.csv')
    r = datafilter(foldername + 'r/' + photfname + '_check.csv')
    i = datafilter(foldername + 'i/' + photfname + '_check.csv')
    z = datafilter(foldername + 'z_s/' + photfname + '_check.csv')
    
    g.doplot(reportfolder)
    r.doplot(reportfolder)
    i.doplot(reportfolder)
    z.doplot(reportfolder)
    
    
    gmr = color(g, r, g.ObjDesig, 'gmr')
    rmi = color(r, i, r.ObjDesig, 'rmi')
    imz = color(i, z, i.ObjDesig, 'imz')
    
    gmr.doplot(reportfolder)
    rmi.doplot(reportfolder)
    imz.doplot(reportfolder)

    inforep = {'Obj': r.ObjDesig,
               'StartTimeObs': r.StartTimeObs,
               'g_TotalTimeObs_h': g.TotalTimeObs,
               'g_NImags': g.NImags,
               'g_MeanVal': g.MeanVal,
               'g_AmplMax': g.AmplMax,
               'g_ErrPerObs': g.MedErrPerObs,
               'r_TotalTimeObs_h': r.TotalTimeObs,
               'r_NImags': r.NImags,
               'r_MeanVal': r.MeanVal,
               'r_AmplMax': r.AmplMax,
               'r_ErrPerObs': r.MedErrPerObs,
               'i_TotalTimeObs_h': i.TotalTimeObs,
               'i_NImags': i.NImags,
               'i_MeanVal': i.MeanVal,
               'i_AmplMax': i.AmplMax,
               'i_ErrPerObs': i.MedErrPerObs,
               'z_TotalTimeObs_h': z.TotalTimeObs,
               'z_NImags': z.NImags,
               'z_MeanVal': z.MeanVal,
               'z_AmplMax': z.AmplMax,
               'z_ErrPerObs': z.MedErrPerObs,
               'gmr_MedColor': gmr.MedColor,
               'gmr_ColorErr': gmr.ColorErr,
               'gmr_AmplMax': gmr.AmplMax,
               'gmr_ErrPerObs': gmr.MedErrPerObs,
               'rmi_MedColor': rmi.MedColor,
               'rmi_ColorErr': rmi.ColorErr,
               'rmi_AmplMax': rmi.AmplMax,
               'rmi_ErrPerObs': rmi.MedErrPerObs,
               'imz_MedColor': imz.MedColor,
               'imz_ColorErr': imz.ColorErr,
               'imz_AmplMax': imz.AmplMax,
               'imz_ErrPerObs': imz.MedErrPerObs,
               }
    return pd.DataFrame(data=inforep, index=[0], columns=inforep.keys())


foldername = '/home/marcel/Templates/211101/'

myinforep = prepareinforeport (foldername + '2004_UE/')
myinforep = myinforep.append(prepareinforeport(foldername + '4660/'))

reportfolder = '/'.join(foldername.split('/')[:-1]) + '/Report_' + \
               foldername.split('/')[-2] + '/'
myinforep.to_csv(reportfolder + 'Report.csv', index=False)




